// This file contains definitions useful for for communicating between
// user mode and the RightSpeed kernel drivers.

#ifndef _RIGHTSPEED_COMMON_KERNEL_USER_H
#define _RIGHTSPEED_COMMON_KERNEL_USER_H

#include "Logging.h"

////////////////////////////////////////////////////////////////////////
// DEVICE TYPES
////////////////////////////////////////////////////////////////////////

#define FILE_DEVICE_RS_INIT             0x0000A83A
#define FILE_DEVICE_RS_LOG              0x0000A83B
#define FILE_DEVICE_RS_HOOK             0x0000A83C
#define FILE_DEVICE_RS_IOCNT            0x0000A83D
#define FILE_DEVICE_RS_TASK             0x0000A83E

////////////////////////////////////////////////////////////////////////
// I/O CONTROL CODES
////////////////////////////////////////////////////////////////////////

#define RS_INIT_IOCTL_GET_SHARED_STATE \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_INIT, 0x00, METHOD_NEITHER, \
                          FILE_ANY_ACCESS)
#define RS_INIT_IOCTL_GET_KERNEL_USER_SHARED_STATE \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_INIT, 0x01, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_INIT_IOCTL_LOG_EVENT \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_INIT, 0x02, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_INIT_IOCTL_WRITE_SPEED_MSR \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_INIT, 0x03, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_INIT_IOCTL_GET_FID_VID_STATUS \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_INIT, 0x04, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_INIT_IOCTL_READ_POWERNOW_TABLE \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_INIT, 0x05, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)

#define RS_LOG_IOCTL_LOG_EVENT \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_LOG, 0x00, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_LOG_IOCTL_STOP_LOGGING \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_LOG, 0x01, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_LOG_IOCTL_START_LOGGING \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_LOG, 0x02, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_LOG_IOCTL_START_NEW_LOG_FILE \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_LOG, 0x03, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_LOG_IOCTL_TERMINATE_CURRENT_LOG_FILE \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_LOG, 0x04, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_LOG_IOCTL_FLUSH \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_LOG, 0x05, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_LOG_IOCTL_RELOAD_REGISTRY \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_LOG, 0x06, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)

#define RS_HOOK_IOCTL_ADD_HOOKS \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_HOOK, 0x00, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_HOOK_IOCTL_REMOVE_HOOKS \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_HOOK, 0x01, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_HOOK_IOCTL_SET_HOOKS \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_HOOK, 0x02, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_HOOK_IOCTL_QUERY_HOOKS \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_HOOK, 0x03, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)

#define RS_TASK_IOCTL_END_TASK_AND_BEGIN_ANOTHER \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_TASK, 0x00, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_TASK_IOCTL_BEGIN_TASK \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_TASK, 0x01, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_TASK_IOCTL_END_TASK \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_TASK, 0x02, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_TASK_IOCTL_SET_GLOBAL_POLICY \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_TASK, 0x03, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_TASK_IOCTL_QUERY_DEADLINE \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_TASK, 0x04, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_TASK_IOCTL_CHANGE_DEADLINE \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_TASK, 0x05, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_TASK_IOCTL_RESET_SAMPLE \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_TASK, 0x06, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_TASK_IOCTL_RESET_ALL_SAMPLES \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_TASK, 0x07, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_TASK_IOCTL_ALLOW_FILE_MODIFICATION \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_TASK, 0x08, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_TASK_IOCTL_FLUSH \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_TASK, 0x09, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_TASK_IOCTL_FLUSH_ALL \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_TASK, 0x0A, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_TASK_IOCTL_QUERY_TTG_FILE_PREAMBLE \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_TASK, 0x0B, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_TASK_IOCTL_QUERY_NUM_SPEEDS \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_TASK, 0x0C, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_TASK_IOCTL_QUERY_TTG_DIRECTORY \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_TASK, 0x0D, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_TASK_IOCTL_PREPARE_FOR_UNINSTALL \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_TASK, 0x0E, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_TASK_IOCTL_GET_FUNCTION_POINTER \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_TASK, 0x0F, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_TASK_IOCTL_QUERY_AVAILABLE_SPEEDS_KHZ \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_TASK, 0x10, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_TASK_IOCTL_QUERY_IDLE_THREAD_TIME \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_TASK, 0x11, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)

#define RS_IOCNT_IOCTL_FILTER_DRIVE \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_IOCNT, 0x00, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)
#define RS_IOCNT_IOCTL_DETACH_FROM_FILE_SYSTEMS \
        (ULONG) CTL_CODE (FILE_DEVICE_RS_IOCNT, 0x01, METHOD_BUFFERED, \
                          FILE_ANY_ACCESS)

////////////////////////////////////////////////////////////////////////
// KERNEL-USER SHARED STATE
////////////////////////////////////////////////////////////////////////

// This structure is shared between kernel and user mode for efficient
// communication:

typedef struct _kernel_user_shared_state {

  // This contains the set of log event categories that are currently
  // being logged by RSLog:
  volatile RS_EVENT_CATEGORY_SET loggedEventCategorySet;

} KERNEL_USER_SHARED_STATE;

#endif
