#ifndef _INSTALL_SPEEDS_H
#define _INSTALL_SPEEDS_H

#include <list>
using namespace std;

// This class represents a single available speed on this CPU.

struct AvailableSpeedInfo {
  AvailableSpeedInfo()
    { }
  AvailableSpeedInfo(DWORD fidVidCodeVal,
                     DWORD reportedMHzVal,
                     DWORD mvVal) :
    fidVidCode(fidVidCodeVal),
    reportedMHz(reportedMHzVal),
    mv(mvVal),
    khz(0)
  {
  }

  // This is the code for this speed:
  DWORD fidVidCode;

  // This is the CPU-reported speed in MHz:
  DWORD reportedMHz;

  // This is the CPU-reported voltage in mV:
  DWORD mv;

  // This is the speed in KHz as computed by a CPU loop:
  DWORD khz;
};

// This class represents a single worthwhile setting on this CPU.

struct WorthwhileSettingInfo {
  WorthwhileSettingInfo(short speedIndexVal,
                        DWORD khzVal,
                        DWORD mvVal) :
    speedIndex(speedIndexVal), khz(khzVal), mv(mvVal)
    { }


  // This is the index into the available speeds array:
  short speedIndex;

  // This is the speed in KHz as computed by a CPU loop:
  DWORD khz;

  // This is the CPU-reported voltage in mV:
  DWORD mv;
};

BOOL GetAvailableSpeeds (list<AvailableSpeedInfo>& availableSpeeds);
BOOL GetWorthwhileSettings (list<WorthwhileSettingInfo>& worthwhileSettings,
                            list<AvailableSpeedInfo>& availableSpeeds);

#endif
