// This file contains functions for sleeping.

#include <ntddk.h>
#include "../Common/Drivers.h"
#include "Globals.h"
#include "Sleep.h"

///////////////////////////////////////////////////////////////////////////
// MACROS
///////////////////////////////////////////////////////////////////////////

// The number of hundreds of nanoseconds per millisecond:

#define HNS_PER_MS               10000

///////////////////////////////////////////////////////////////////////////
// SLEEPING
///////////////////////////////////////////////////////////////////////////

// This function sleeps the current thread for the given number of
// milliseconds.

void Sleep (long milliseconds)
{
  LARGE_INTEGER timeout;
  KEVENT neverEvent;

  // Convert the milliseconds to hundreds of nanoseconds, and also make
  // it negative to indicate a time relative to now.

  timeout.QuadPart = milliseconds;
  timeout.QuadPart *= -HNS_PER_MS;

  // Wait for an event that will never be signalled, with the given
  // timeout.
  
  KeInitializeEvent(&neverEvent, SynchronizationEvent, FALSE);
  KeWaitForSingleObject(&neverEvent, Executive, KernelMode, TRUE,
                        &timeout);

  DebugPrint(("RSIoCnt: Waited for %ld ms.\n", milliseconds));
}
