// This file contains information about global variables in the RSLog
// driver.

#include "../Common/DrvShare.h"
#include "Buffer.h"
#include "LogFile.h"
#include "RegInfo.h"

////////////////////////////////////////////////////////////////////////////
//                                MACROS
////////////////////////////////////////////////////////////////////////////

// The following flags are used to keep track of which resources have
// been allocated so far for the RSLog driver. So, when we exit, we
// can free these resources.

#define RSINIT_FILE_OBJECT_RESOURCE_FLAG                1
#define SHARED_STATE_HOOKS_RESOURCE_FLAG                2
#define SYMBOLIC_LINK_RESOURCE_FLAG                     4
#define INTERFACE_DEVICE_RESOURCE_FLAG                  8
#define LOG_BUFFER_RESOURCE_FLAG                        16
#define LOG_FILE_RESOURCE_FLAG                          32
#define REGISTRY_INFO_RESOURCE_FLAG                     64

////////////////////////////////////////////////////////////////////////////
//                             STRUCTURES
////////////////////////////////////////////////////////////////////////////

// The structure RS_LOG_GLOBALS contains all the global variables
// used by this driver in one place, so it's clear what's global and
// what isn't.

typedef struct _rs_log_globals {

  // This field describes all the resources that must be freed before
  // exiting:
  ULONG                        resourcesObtained;
  // This is a pointer to our driver object:
  PDRIVER_OBJECT               ourDriver;
  // This is a pointer to our interface device object:
  PDEVICE_OBJECT               interfaceDeviceObject;
  // This is the configuration information read from the registry:
  REGISTRY_INFO                registryInfo;

  // This is a pointer to the state shared with all RightSpeed drivers:
  SHARED_STATE                 *sharedState;
  // This is a file object referencing RSInit so sharedState stays put:
  PFILE_OBJECT                 RSInitFileObject;
  // The log-event function we replaced:
  LOG_FUNC_TYPE                replacedLogEventFunc;

  // Each bit in the following indicates a reason why logging is
  // currently disabled, so logging is enabled if and only if this
  // field has value 0:
  ULONG                        whyLoggingDisabled;
  // Information about the log buffer:
  LOG_BUFFER_INFO              logBufferInfo;
  // Information about the log file:
  LOG_FILE_INFO                logFileInfo;

} RS_LOG_GLOBALS;

/////////////////////////////////////////////////////////////////////////
//                         EXTERNAL VARIABLES
/////////////////////////////////////////////////////////////////////////

extern RS_LOG_GLOBALS g;

