#ifndef _FILETIME_H
#define _FILETIME_H

// This file contains the declaration of the FileTime class, which
// represents an NT time. It can be absolute or relative, it can be
// infinite, and it can be known or unknown.

#include <iostream>
#include "genio.h"
#include "nanosec.h"

namespace Windows {

  // The declaration of the FileTime class:

  class FileTime {
  public:
    FileTime() : known(false), absolute(false), infinite(false), time(0) { }
    FileTime(LONGLONG hnsSince1601, int localUTCOffset,
             bool daylightUsed);
    FileTime(unsigned long milliseconds);
    FileTime(char ntTime[8]);
    ~FileTime() { }

    bool IsKnown () { return known; }
    bool IsAbsolute () { return known && absolute; }
    bool IsInfinite () { return known && infinite; }

    void Invalidate () { known = false; }

    jrl::Nanosec GetTime ();

    void Print (ostream& os) const;
    void Write (jrl::RawOutput& od) const;
    void Read (jrl::RawInput& is);

  private:
    bool known;        // Is the time known?
    bool absolute;     // Is the time absolute (as opposed to relative)?
    bool infinite;     // Is the time infinite?
    jrl::Nanosec time;      // Time in nanoseconds
  };

  // The following functions deal with input and output of FileTime's:

  inline ostream& operator<< (ostream& os, const FileTime& f)
  { f.Print(os); return os; }
  inline jrl::RawOutput& operator<< (jrl::RawOutput& od, const FileTime& f)
  { f.Write(od); return od; }
  inline jrl::RawInput& operator>> (jrl::RawInput& is, FileTime& f)
  { f.Read(is); return is; }

}

#endif
