#ifndef _VTRACE_TIMES_H
#define _VTRACE_TIMES_H

#include <windows.h>
#include <iostream>
#include "genio.h"
#include "typeconv.h"

namespace jrl {

  class Time {
  public:
    Time();
    Time(unsigned long seconds, unsigned long nanoseconds);
    Time(double seconds);
    Time(LONGLONG cycles, unsigned long cpuSpeed);
    Time(LONGLONG cycles, unsigned long cpuSpeed, const Time& offset);
    Time(unsigned long cyclesHigh, unsigned long cyclesLow,
         unsigned long cpuSpeed);
    Time(const Time& time);
    ~Time() { }
    void SetFields (unsigned long seconds, unsigned long nanoseconds);
    void GetFields (unsigned long& seconds, unsigned long& nanoseconds) const;
    Time& operator+= (const Time& addend);
    Time operator+ (const Time& addend) const;
    Time& operator-= (const Time& subtraend);
    Time operator- (const Time& subtraend) const;
    bool operator>= (const Time& other) const;
    bool operator> (const Time& other) const;
    bool operator<= (const Time& other) const;
    bool operator< (const Time& other) const;
    bool operator== (const Time& other) const;
    bool operator!= (const Time& other) const;
    string AsString () const;
    double ConvertToDouble () const;
    unsigned long RoundToMicroseconds () const;
    unsigned long RoundToMilliseconds () const;
    unsigned long RoundToSeconds () const;
    void Print (ostream& os) const;
    void Write (jrl::RawOutput& od) const { od << sec << ns; }
    void Read (jrl::RawInput& is) { is >> sec >> ns; }
  
  private:
    unsigned long sec;
    long ns;
  };
  
  Time MaxTime ();
  Time ConvertMillisecondsToTime (unsigned long milliseconds);

  ostream& operator<< (ostream& os, const Time& t);
  ostream& operator<< (ostream& os, Time *t);
  jrl::RawInput& operator>> (jrl::RawInput& is, Time& t);
  jrl::RawOutput& operator<< (jrl::RawOutput& od, const Time& t);
}
  
#endif
