// This file contains information about global variables in the RSTask
// driver.

#include "../Common/DrvShare.h"
#include "RegInfo.h"
#include "TimerRes.h"
#include "Flush.h"
#include "TTGFile.h"
#include "SampleQ.h"
#include "IdleThread.h"
#include "SpeedControl.h"
#include "Tasks.h"

////////////////////////////////////////////////////////////////////////////
//                                MACROS
////////////////////////////////////////////////////////////////////////////

// The following flags are used to keep track of which resources have
// been allocated so far for the RSLog driver. So, when we exit, we
// can free these resources.

#define RSINIT_FILE_OBJECT_RESOURCE_FLAG                1
#define TIMER_RESOLUTION_RESOURCE_FLAG                  2
#define TTG_DIRECTORY_RESOURCE_FLAG                     4
#define AVAILABLE_SPEEDS_INFO_RESOURCE_FLAG             8
#define WORTHWHILE_SETTINGS_INFO_RESOURCE_FLAG          16
#define SAMPLE_QUEUE_RESOURCE_FLAG                      32
#define OPEN_TTG_FILES_INFO_RESOURCE_FLAG               64
#define SPEED_CONTROLLER_RESOURCE_FLAG                  128
#define ONGOING_TASKS_RESOURCE_FLAG                     256
#define IDLE_THREAD_RESOURCE_FLAG                       512
#define PERIODIC_FLUSH_RESOURCE_FLAG                    1024
#define SHARED_STATE_HOOKS_RESOURCE_FLAG                2048
#define INTERFACE_DEVICE_RESOURCE_FLAG                  4096
#define SYMBOLIC_LINK_RESOURCE_FLAG                     8192
#define SHUTDOWN_NOTIFICATION_RESOURCE_FLAG             16384
#define SPEED_CURVES_RESOURCE_FLAG                      32768

////////////////////////////////////////////////////////////////////////////
//                             STRUCTURES
////////////////////////////////////////////////////////////////////////////

// The structure RS_TASK_GLOBALS contains all the global variables
// used by this driver in one place, so it's clear what's global and
// what isn't.

typedef struct _rs_task_globals {

  // This field describes all the resources that must be freed before
  // exiting:
  ULONG                        resourcesObtained;
  // This is a pointer to our driver object:
  PDRIVER_OBJECT               ourDriver;
  // This is a pointer to our interface device object:
  PDEVICE_OBJECT               interfaceDeviceObject;

  // This is a pointer to the state shared with all RightSpeed drivers:
  SHARED_STATE                 *sharedState;
  // This is a file object referencing RSInit so sharedState stays put:
  PFILE_OBJECT                 RSInitFileObject;
  // The notify-I/Os-complete function we replaced:
  SIMPLE_FUNC_TYPE             replacedNotifyIosCompleteFunc;

  // This is the configuration information read from the registry:
  REGISTRY_INFO                registryInfo;

  // This is the timer resolution information:
  TIMER_RESOLUTION_INFO        timerResolutionInfo;

  // The information about periodic flush of TTG files:
  PERIODIC_FLUSH_INFO          periodicFlushInfo;

  // This information is used to keep track of open TTG files:
  OPEN_TTG_FILES_INFO          openTTGFilesInfo;

  // This information describes the sample value queue:
  SAMPLE_QUEUE_INFO            sampleQueueInfo;

  // This information describes the idle thread:
  IDLE_THREAD_INFO             idleThreadInfo;

  // This information describes all ongoing tasks:
  ONGOING_TASKS_INFO           ongoingTasksInfo;

  // This information describes the current state of the speed controller:
  SPEED_CONTROLLER_INFO        speedControllerInfo;

} RS_TASK_GLOBALS;

/////////////////////////////////////////////////////////////////////////
//                         EXTERNAL VARIABLES
/////////////////////////////////////////////////////////////////////////

extern RS_TASK_GLOBALS g;


