// This file contains I/O-counting callback functions.

#include <ntddk.h>
#include "Globals.h"
#include "IdleThread.h"
#include "SpeedControl.h"

// This function is called whenever the last ongoing I/O completes.
// It resumes any paused schedule, and signals the idle thread to wake
// up and end any ongoing tasks when the system next becomes idle.

void NotifyIosComplete (void)
{
  // Resume any paused schedule.

  ResumeSchedule();

  // Signal the idle thread to wake up and end any ongoing tasks. We
  // have to do this because it may have "wasted" its signal to wake
  // up on waking up when I/O's were ongoing. There is, however, no
  // point in waking up the idle thread if there aren't any ongoing
  // tasks, so don't bother unless there are ongoing tasks.
  
  if (g.sharedState->numOngoingTasks)
    KeSetEvent(&g.idleThreadInfo.threadWaitEvent, IO_NO_INCREMENT, FALSE);
}
