// This file contains definitions relevant to obtaining data from the
// registry.

#ifndef _RSTASK_REGISTRY_INFO_H
#define _RSTASK_REGISTRY_INFO_H

/////////////////////////////////////////////////////////////////////////////
// MACROS
/////////////////////////////////////////////////////////////////////////////

#define INDEX_OF_SETTING(setting) \
   (g.registryInfo.worthwhileSettingsInfo[setting].speedIndex)
#define CPI_OF_SETTING(setting) \
   (g.registryInfo.worthwhileSettingsInfo[setting].cpi)
#define CPI_OF_INDEX(speedIndex) \
   (g.registryInfo.availableSpeedsInfo[speedIndex].cpi)
#define CYC_PER_MS_OF_INDEX(speedIndex) \
   (g.registryInfo.availableSpeedsInfo[speedIndex].cycPerMs)
#define CYC_PER_MS_OF_SETTING(setting) \
   CYC_PER_MS_OF_INDEX(INDEX_OF_SETTING(setting))

#define SPEED_CURVE_VALUE(whichCurve, whichValue) \
   (ULONG) (g.registryInfo.speedCurves[whichCurve * \
                                       g.registryInfo.speedCurveLength + \
                                       whichValue])

/////////////////////////////////////////////////////////////////////////////
// STRUCTURES
/////////////////////////////////////////////////////////////////////////////

// This structure contains information about a CPU speed available on
// this CPU:

typedef struct _available_speed_info {

  // The number of cycles per millisecond:
  ULONG cycPerMs;

  // The number of cycles per scheduling interval:
  double cpi;

  // The code for this speed:
  ULONG speedCode;
  
} AVAILABLE_SPEED_INFO, *PAVAILABLE_SPEED_INFO;


// This structure contains information about a CPU speed available on
// this CPU and worthwhile to use:

typedef struct _worthwhile_setting_info {

  // The number of cycles per scheduling interval:
  double cpi;
  
  // The corresponding index into the available speed array:
  ULONG speedIndex;

} WORTHWHILE_SETTING_INFO, *PWORTHWHILE_SETTING_INFO;

// This structure contains information gleaned from the registry:

typedef struct _registry_info {

  // The period, in seconds, of the flushing of all open TTG files:
  ULONG ttgFlushPeriodSec;

  // The sample queue size:
  ULONG sampleQueueSize;

  // The TTG directory:
  UNICODE_STRING ttgDirectory;

  // The last time registry information about this CPU was modified:
  LARGE_INTEGER cpuInfoModificationTime;

  // The number of available speeds on this CPU:
  short numAvailableSpeeds;

  // Information about the available speeds on this CPU:
  PAVAILABLE_SPEED_INFO availableSpeedsInfo;

  // The number of worthwhile speed settings on this CPU:
  short numWorthwhileSettings;

  // Information about the worthwhile settings on this CPU:
  PWORTHWHILE_SETTING_INFO worthwhileSettingsInfo;

  // The index of the highest available speed:
  ULONG maxAvailableSpeedIndex;

  // The codes for the minimum and maximum available speeds:
  ULONG minSpeedCode;
  ULONG maxSpeedCode;

  // The minimum and maximum available speeds, in cycles per interval:
  double minAvailableCpi;
  double maxAvailableCpi;

  // The highest worthwhile setting:
  short maxWorthwhileSetting;

  // The speed indexes corresponding to the lowest and highest worthwhile
  // settings:
  ULONG minWorthwhileSpeedIndex;
  ULONG maxWorthwhileSpeedIndex;

  // The speeds, in cycles per second, corresponding to the lowest and
  // highest worthwhile settings:
  double minWorthwhileCpi;
  double maxWorthwhileCpi;

  // The speeds, in cycles per millisecond, corresponding to the
  // lowest and highest worthwhile settings:
  ULONG minWorthwhileCycPerMs;
  ULONG maxWorthwhileCycPerMs;

  // The speed curves that PACE should consider, and the number of
  // such curves:
  unsigned char *speedCurves;
  ULONG speedCurveLength;
  ULONG numSpeedCurves;

} REGISTRY_INFO;

NTSTATUS ReadRegistryInfo (REGISTRY_INFO *info);
void FreeRegistryInfo (REGISTRY_INFO *info);
NTSTATUS ReloadRegistryInfo (REGISTRY_INFO *info);

#endif
