// This file contains definitions relating to the queue of sample values.

#ifndef _RSTASK_SAMPLE_QUEUE_H
#define _RSTASK_SAMPLE_QUEUE_H

#include "../Common/TTG.h"
#include "TTGFile.h"

/////////////////////////////////////////////////////////////////////////////
// STRUCTURES
/////////////////////////////////////////////////////////////////////////////

// This structure describes information about a single sample value that
// has to be added to a sample.

typedef struct _sample_queue_entry {
  PTTG_FILE ttgFile;
  PTT_INFO ttInfo;
  LONGLONG cycles;
} SAMPLE_QUEUE_ENTRY, *PSAMPLE_QUEUE_ENTRY;

// This structure describes information about the sample queue.

typedef struct _sample_queue {

  // This spin lock protects access to this structure and all the
  // entries in the queue:
  KSPIN_LOCK spinLock;

  // This is an array of all the entries allocated in the queue,
  // including currently unused ones:
  PSAMPLE_QUEUE_ENTRY entries;

  // This is the size of the 'entries' array, the maximum number
  // of sample values that it can hold:
  LONG maxSize;

  // This is the index in the 'entries' array of the first free entry:
  LONG next;

  // This is the number of used entries in the 'entries' array, i.e.,
  // the number of unprocessed sample values in the queue:
  LONG size;

} SAMPLE_QUEUE_INFO, *PSAMPLE_QUEUE_INFO;

/////////////////////////////////////////////////////////////////////////////
// FUNCTION PROTOTYPES
/////////////////////////////////////////////////////////////////////////////

NTSTATUS InitializeSampleQueue (PSAMPLE_QUEUE_INFO info);
void CloseSampleQueue (PSAMPLE_QUEUE_INFO info);

void AddSampleValueToQueue (
  LONGLONG cycles,
  PTT_INFO ttInfo,
  PTTG_FILE ttgFile
  );
void ProcessSamplesInQueue (void);

#endif
