// This file contains my Unicode functions.

#include <ntddk.h>
#include "Unicode.h"

////////////////////////////////////////////////////////////////////////
//                           UNICODE STRINGS
////////////////////////////////////////////////////////////////////////

// MyAllocateUnicodeString allocates space for a Unicode string of at
// most maxLength bytes, and fills in a Unicode string structure
// pointed to by string that refers to that space. It returns a status
// value.

NTSTATUS MyAllocateUnicodeString (PUNICODE_STRING string, USHORT maxLength)
{
  string->Buffer = ExAllocatePool(NonPagedPool, maxLength);
  if (string->Buffer == NULL)
    return STATUS_INSUFFICIENT_RESOURCES;
  string->Length = 0;
  string->MaximumLength = maxLength;

  return STATUS_SUCCESS;
}

// MyFreeUnicodeString deallocates space that was allocated for a given
// Unicode string by MyAllocateUnicodeString.

void MyFreeUnicodeString (PUNICODE_STRING string)
{
  if (string->Buffer != NULL) {
    ExFreePool(string->Buffer);
    string->Buffer = NULL;
    string->Length = 0;
    string->MaximumLength = 0;
  }
}

// MyClearUnicodeString sets a Unicode string to the empty string.

void MyClearUnicodeString (PUNICODE_STRING string)
{
  string->Length = 0;
}
