// This file contains the implementations for the global classes.

#include <windows.h>
#include "Globals.h"
#include "Log.h"

////////////////////////////////////////////////////////////////////////////
// GLOBAL VARIABLE INSTANTIATION
////////////////////////////////////////////////////////////////////////////

// This instantiates the systemwide global object, and puts it in the
// .shared segment so that it is shared by all processes:

#pragma data_seg(".shared")
SystemwideGlobals systemwideGlobals = { false, false, false };
#pragma data_seg()

// This instantiates the per-process global object:

Globals g;

////////////////////////////////////////////////////////////////////////////
// GLOBALS CLASS
////////////////////////////////////////////////////////////////////////////

// This function initializes an object in the Globals class. It sets
// its 'initializedOK' field to a boolean indicating whether it was
// successful.

Globals::Globals () :
  initializedOK(false)
{
  if (Initialize())
    initializedOK = true;
  else {
    // If we couldn't completely initialize, close any parts that were
    // initialized.
    Close();
  }
}

// This function tries to initialize all the fields in the Globals
// object, and returns a boolean indicating if it was successful.

bool Globals::Initialize ()
{
  // Initialize the handle to the RSInit device.

  if (!initDeviceHandle.Initialize())
    return false;

  // Initialize the hooks information.

  if (!hooks.Initialize())
    return false;

  // Enable hooks for this process.

  hooks.Enable();

  // Log the successful initialization for this process.

  if (LOGGING_INFORMATION)
    LOG_ERROR_NO_PARAMETERS(RS_LIB_SUCCESS_INITIALIZED_OK);

  // Return true indicating we successfully initialized.

  return true;
}

// This function closes all resources associated with this object.

void Globals::Close ()
{
  initializedOK = false;
  hooks.Close();
  initDeviceHandle.Close();
}
