// This file defines classes related to hooking threads' message queues.

#ifndef _RSLIB_HOOKS_H
#define _RSLIB_HOOKS_H

#include <list>
#include "RSTaskHandle.h"

// This class describes all the hooks associated with a single thread.

class ThreadHooks {
public:
  ThreadHooks(DWORD threadIdVal);
  ~ThreadHooks() { }

  DWORD GetThreadId () { return threadId; }
  void Enable ();
  void Disable ();

private:
  // This boolean indicates whether this thread's hooks are enabled:
  bool enabled;

  // This is the ID of the thread this object refers to:
  DWORD threadId;

  // This is the handle to the message hook for this thread, if hooking
  // is enabled:
  HHOOK hMessageHook;
};

// This class describes associated with this process. It also keeps
// track of the thread ID's of all active threads, so that it can enable
// or disable hooking for all such threads.

class Hooks {
public:
  Hooks();
  ~Hooks() { Close(); }

  bool Initialize ();
  void Close ();

  void NotifyThreadStarting (DWORD threadId);
  void NotifyThreadEnding (DWORD threadId);

  void Enable ();
  void Disable ();

  void HandleMessage (bool removed, MSG *msg);

  void TemporarilyEnable () { functionalityEnabled = true; }
  void TemporarilyDisable () { functionalityEnabled = false; }

private:
  // This critical section controls access to all fields of this
  // object:
  CRITICAL_SECTION criticalSection;

  // This is a list of thread hooks information objects, one for each
  // thread in this process:
  list<ThreadHooks> threadHooks;

  // This is a handle to the systemwide task type group file for
  // automatic tasks:
  RSTaskHandle hSystemTasks;

  // This is a handle to the per-application task type group file for
  // automatic tasks:
  RSTaskHandle hAppTasks;

  // This boolean indicates whether hooking is enabled for this process:
  bool enabled;

  // This boolean indicates whether the hooks should actually do anything
  // when they trigger:
  bool functionalityEnabled;

  // If the last message hook call retrieved a message without
  // removing it from the queue, this field points to that message;
  // otherwise, this field is 0:
  MSG *lastUnremovedMessagePtr;

  // This is the task ID of the last automatic task we started:
  DWORD lastAutomaticTaskId;
};

#endif
