// This file defines the RSInitHandle class, which represents an open
// handle to the RSInit device.

#ifndef _RSLIB_RSINIT_HANDLE_H
#define _RSLIB_RSINIT_HANDLE_H

#include "../Common/Logging.h"
#include "../Common/KernelUser.h"
#include <string>
using namespace std;

// This class represents an open handle to the RSInit device.

class RSInitHandle {
public:
  RSInitHandle();
  ~RSInitHandle() { Close(); }

  bool Initialize ();
  void Close ();

  // These functions indicate whether logging is on for a particular
  // category of event type. They directly check the shared
  // kernel-user state.

  bool IsLoggingTasks ()
  { return sharedState &&
    (sharedState->loggedEventCategorySet & RS_EVENT_CATEGORY_TASKS) != 0; }
  bool IsLoggingErrors ()
  { return sharedState &&
    (sharedState->loggedEventCategorySet & RS_EVENT_CATEGORY_ERRORS) != 0; }
  bool IsLoggingWarnings ()
  { return sharedState &&
    (sharedState->loggedEventCategorySet & RS_EVENT_CATEGORY_WARNINGS) != 0; }
  bool IsLoggingInformation ()
  { return sharedState &&
    (sharedState->loggedEventCategorySet & RS_EVENT_CATEGORY_INFORMATION)
      != 0; }

  void LogString (const string& s);

  void LogError (DWORD errorCode);
  void LogErrorOneParameter (DWORD errorCode, DWORD param);
  void LogErrorTwoParameters (DWORD errorCode, DWORD param1, DWORD param2);

  void LogAutoSystemTask ();
  void LogAutoAppTask (DWORD taskType);
  void LogProcessHooked (char *processName);
  void LogThreadHooked (DWORD threadId);
  void LogThreadUnhooked (DWORD threadId);
  void LogActionStarted (DWORD actionType);
  void LogActionStopped (DWORD actionType);
  
  void LogBytes (short entryType, LPCVOID data, DWORD dataLength);

private:
  // This is the handle to the RSInit device:
  HANDLE h;

  // This is the pointer to the kernel-user shared state that we get
  // from the RSInit device. It's valid as long as we have the handle
  // open:
  KERNEL_USER_SHARED_STATE *sharedState;
};

#endif
