// This file defines the class describing a task type's parameters.

#ifndef _RSLIB_TASK_TYPE_H
#define _RSLIB_TASK_TYPE_H

#include "../Common/Algorithm.h"

// This class describes a task type's parameters, as deduced from a task
// type description passed to a call to create a task type group file.

class TaskType {
public:
  TaskType();
  ~TaskType() { }

  bool ApplyDescription (const char *description, DWORD maxSpeed);

  unsigned long GetDeadlineHns () const;
  BOOLEAN GetUseSample () const;
  BOOLEAN GetSampleIfIo () const { return sampleIfIo; }
  long GetAlgorithmType () const { return algorithmType; }

  int GetAlgorithmDataSize () const;
  void InitializeAlgorithmData (PVOID algorithmDataAddr) const;

private:
  void InitializeAgBasicData (PAG_BASIC_DATA data) const;
  
  // This is the type of algorithm to use to compute schedules:
  long algorithmType;

  // This is the deadline in milliseconds:
  double deadline;

  // This boolean indicates whether we should include in the sample
  // those tasks that perform I/O:
  BOOLEAN sampleIfIo;

  // This is the average pre-deadline speed, in MHz, to use:
  double speed;

  // This is the number of sample values that must occur before the first
  // reschedule:
  int sampleValuesBeforeFirstReschedule;

  // This is the number of sample values that should occur between
  // reschedules:
  int sampleValuesBetweenReschedules;

  // This is the aging factor to use:
  double agingFactor;

  // This is the period, in milliseconds, after which updates to
  // the speed should occur in the schedule:
  double updatePeriod;

  // This is the target deadlines made percentile to use:
  double percentOfDeadlinesToMake;

};

#endif

