// This file contains definitions for sharing data among RightSpeed
// drivers.

#ifndef _RIGHTSPEED_COMMON_DRVSHARE_H
#define _RIGHTSPEED_COMMON_DRVSHARE_H

#include "KernelUser.h"

/////////////////////////////////////////////////////////////////////////
// STRUCTURES
/////////////////////////////////////////////////////////////////////////

// The following are function types of shared functions:

typedef PCHAR (*LOG_FUNC_TYPE) (short, ULONG);
typedef void (*SIMPLE_FUNC_TYPE) (void);

// The SHARED_STATE structure is obtained from RSInit by the other
// drivers so that they can share this state.

typedef struct _shared_state {

  // This spin lock protects access to the shared items in this
  // structure:
  KSPIN_LOCK spinLock;

  // This function places entries into the log:
  volatile LOG_FUNC_TYPE logEventFunc;

  // This field counts the number of ongoing I/O's:
  long numOngoingIos;

  // This routine should be called when the number of ongoing I/O's
  // drops to zero:
  volatile SIMPLE_FUNC_TYPE notifyIosCompleteFunc;

  // This structure contains items that should also be shared with
  // user mode:
  KERNEL_USER_SHARED_STATE kernelUserSharedState;

  // This field counts the number of ongoing tasks:
  long numOngoingTasks;

} SHARED_STATE;

#endif
