// This file contains information about RightSpeed parameters, such as
// their default, minimum, and maximum values.

#ifndef _RIGHTSPEED_COMMON_PARAMETERS_H
#define _RIGHTSPEED_COMMON_PARAMETERS_H

#include "Logging.h"

// Log buffer size, in KB:

#define MIN_LOG_BUFFER_SIZE                   8
#define DEFAULT_LOG_BUFFER_SIZE               128
#define MAX_LOG_BUFFER_SIZE                   4096

// Number of available speeds:

#define MAX_NUM_AVAILABLE_SPEEDS              256

// Length of RightSpeed main directory:

#define MAX_RIGHTSPEED_DIRECTORY_LENGTH       256

// Length of task type group file full path:

#define MAX_TTG_FILE_FULL_PATH_LENGTH         300

// Number of phases in a speed schedule:

#define MAX_NUM_SCHEDULE_PHASES               30

// Period for task type group flushing, in seconds. So, for instance,
// a value of 3600 means to flush all task type groups every hour:

#define MIN_TTG_FLUSH_PERIOD_SEC              300     /* 5 minutes */
#define DEFAULT_TTG_FLUSH_PERIOD_SEC          3600    /* an hour */
#define MAX_TTG_FLUSH_PERIOD_SEC              604800  /* a week */

// The sample queue size indicates how many samples are allowed to
// be pending on the queue at a time:

#define MIN_SAMPLE_QUEUE_SIZE                 0
#define DEFAULT_SAMPLE_QUEUE_SIZE             12
#define MAX_SAMPLE_QUEUE_SIZE                 64

// The number of ideal schedule phases in the PACE computation:

#define IDEAL_SCHEDULE_NUM_PHASES             20

// Parameters of various scheduling strategies:

#define DEFAULT_DEADLINE_MS                                 50
#define DEFAULT_SAMPLER_AGING_FACTOR                        0.95
#define DEFAULT_PREDEADLINE_SPEED_MHZ                       500
#define DEFAULT_UPDATE_PERIOD_MS                            10
#define MAX_UPDATE_PERIOD_MS                                200000
#define DEFAULT_SAMPLE_VALUES_BEFORE_RESCHEDULE             2
#define MIN_SAMPLE_VALUES_BEFORE_RESCHEDULE                 2
#define DEFAULT_SAMPLE_VALUES_BETWEEN_RESCHEDULES           1
#define MIN_SAMPLE_VALUES_BETWEEN_RESCHEDULES               1
#define DEFAULT_PERCENT_OF_DEADLINES_TO_MAKE                99

// Parameters of automatic task detection:

#define AUTO_TASK_MOVE_DESCRIPTION            "speed=200"
#define AUTO_TASK_KEY_DESCRIPTION             "speedRelativeToMaximum=0.7"
#define AUTO_TASK_CLICK_DESCRIPTION           "speedRelativeToMaximum=0.85"

// The number of recent sample values to keep in a sample:

#define MIN_WINDOW_SIZE                       2
#define DEFAULT_WINDOW_SIZE                   28
#define MAX_WINDOW_SIZE                       128

// How many task types can be in a task type group:

#define MAX_NUM_TASK_TYPES                    512

// How many bytes a task type group file can have:

#define MAX_TTG_INFO_SIZE                     524280

// Maximum length in bytes of a description of a task type passed to
// Mkttg:

#define MAX_TASK_TYPE_DESCRIPTOR_LEN          1024

// The default event category set is the one to start using as the set
// to log in the absence of any other information:

#define DEFAULT_RS_EVENT_CATEGORY_SET  (RS_EVENT_CATEGORY_ERRORS | \
                                        RS_EVENT_CATEGORY_WARNINGS)

// These are the registry parameters we use in RightSpeed:
//
// Version
// MainDirectory
// SampleQueueSize
// TTGFlushPeriodSec
// LoggedEventCategorySet
// LogBufferKB
// LastLogNumber
// PaceFittedExponent
// TmtaSpeedCodes
// WorthwhileSettings
// AvailableSpeeds
// TscTicksPerMs
// CpuInfoModificationTime

#endif
