// This file contains definitions useful in setting speed schedules.

#ifndef _RIGHTSPEED_COMMON_SCHEDULE_H
#define _RIGHTSPEED_COMMON_SCHEDULE_H

#include "Parameters.h"

////////////////////////////////////////////////////////////////////////////
// STRUCTURES
////////////////////////////////////////////////////////////////////////////

// A speed schedule phase describes a single phase of a speed schedule,
// giving the index of the speed to use as well as the number of intervals
// the phase should last for. A value of '0' for intervals indicates the
// phase should last indefinitely.

typedef struct _speed_schedule_phase {
  unsigned long speedIndex;
  long intervals;
} SPEED_SCHEDULE_PHASE, *PSPEED_SCHEDULE_PHASE;

// A speed schedule consists of a sequence of speed schedule phases,
// the last of which must be indefinite, i.e., have an intervals field
// value of '0'.

typedef struct _speed_schedule {

  // This field indicates how many phases are in this schedule:
  short numPhases;

  // This field indicates the maximum number of phases that can fit
  // in this data structure, wherever it's allocated:
  short maxNumPhases;

  // This array contains the phases of the speed schedule in order:
  SPEED_SCHEDULE_PHASE phase[1];

} SPEED_SCHEDULE, *PSPEED_SCHEDULE;

// A speed schedule header is a speed schedule that doesn't include
// space for all its phases, but just holds a pointer to a buffer that
// has those phases.

typedef struct _speed_schedule_header {
  short numPhases;
  short maxNumPhases;
  SPEED_SCHEDULE_PHASE *phase;
} SPEED_SCHEDULE_HEADER, *PSPEED_SCHEDULE_HEADER;

// A maximal speed schedule is a speed schedule structure that can hold
// up to MAX_NUM_SCHEDULE_PHASES schedule phases:

typedef struct _maximal_speed_schedule {
  short numPhases;
  short maxNumPhases;     // should always be MAX_NUM_SCHEDULE_PHASES
  SPEED_SCHEDULE_PHASE phase[MAX_NUM_SCHEDULE_PHASES];
} MAXIMAL_SPEED_SCHEDULE, *PMAXIMAL_SPEED_SCHEDULE;

#endif
