// InstallDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Install.h"
#include "InstallDlg.h"
#include "InstallRoutines.h"
#include "../Common/Parameters.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CInstallDlg dialog

CInstallDlg::CInstallDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CInstallDlg::IDD, pParent)
{
  //{{AFX_DATA_INIT(CInstallDlg)
  m_mainDirectory = _T("");
  // NOTE: the ClassWizard will add member initialization here
  //}}AFX_DATA_INIT
  // Note that LoadIcon does not require a subsequent DestroyIcon in Win32
  m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
  installingThread = NULL;
}

void CInstallDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  //{{AFX_DATA_MAP(CInstallDlg)
  DDX_Text(pDX, IDC_EDIT1, m_mainDirectory);
  // NOTE: the ClassWizard will add DDX and DDV calls here
  //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CInstallDlg, CDialog)
  //{{AFX_MSG_MAP(CInstallDlg)
  ON_WM_PAINT()
  ON_WM_QUERYDRAGICON()
  ON_WM_SYSCOMMAND()
  ON_BN_CLICKED(ID_BROWSE, OnBrowse)
  ON_BN_CLICKED(IDC_FREE_SPACE, OnFreeSpace)
  ON_MESSAGE(WM_CHANGE_STATUS, OnChangeStatus)
  ON_MESSAGE(WM_INSTALL_COMPLETE, OnInstallComplete)
  //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CInstallDlg message handlers

BOOL CInstallDlg::OnInitDialog()
{
  HKEY hKey;
  LONG status;
  DWORD type, length, n;
  char mainDirectory[MAX_RIGHTSPEED_DIRECTORY_LENGTH];
  CString programFilesDirectory;

  // Initialize registry variables to reasonable values in case they
  // cannot be found in the registry

  if (GetProgramFilesDirectory(programFilesDirectory))
    m_mainDirectory = programFilesDirectory + "\\RightSpeed";
  else
    m_mainDirectory = "C:\\RightSpeed";
  m_previousVersion = 0;
  m_formerMainDirectory = "";

  // Search the registry for information

  status = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
                        "Software\\Jay Lorch\\RightSpeed\\CurrentVersion",
                        0, KEY_QUERY_VALUE, &hKey);
  if (status == ERROR_SUCCESS) {
    length = MAX_RIGHTSPEED_DIRECTORY_LENGTH;
    status = RegQueryValueEx(hKey, "MainDirectory", NULL, &type,
                             (BYTE *) mainDirectory, &length);
    if (status == ERROR_SUCCESS && type == REG_SZ) {
      m_mainDirectory = mainDirectory;
      m_formerMainDirectory = mainDirectory;
    }

    length = 4;
    status = RegQueryValueEx(hKey, "Version", NULL, &type, (BYTE *) &n,
                             &length);
    if (status == ERROR_SUCCESS && type == REG_DWORD && length == 4)
      m_previousVersion = n;

    RegCloseKey(hKey);
  }

  CDialog::OnInitDialog();

  // Set the icon for this dialog.  The framework does this automatically
  //  when the application's main window is not a dialog
  SetIcon(m_hIcon, TRUE);     // Set big icon
  SetIcon(m_hIcon, FALSE);    // Set small icon
  
  // TODO: Add extra initialization here
  
  return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CInstallDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CInstallDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

// CInstallDlg::OnOK is called when the OK button is clicked.  It launches a
// thread to install RightSpeed.

void CInstallDlg::OnOK() 
{
  CWnd *okButton;

  // Update the dialog object data to reflect the window

  UpdateData(TRUE);

  if (installingThread == NULL) {
    if (CheckFields()) {
      // Disable the OK button
      okButton = GetDlgItem(IDOK);
      if (okButton != NULL) okButton->EnableWindow(FALSE);

      // Launch the install thread and pass it the appropriate parameters
      installParameters.hWnd = m_hWnd;
      installParameters.mainDirectory = m_mainDirectory;
      installParameters.previousVersion = m_previousVersion;
      installingThread = AfxBeginThread(&InstallAsynchronously,
                                        &installParameters);
    }
  }
}

// CInstallDlg::OnCancel is called when the Cancel button is clicked.  It
// terminates the installing thread (if it is running), then ends the program.

void CInstallDlg::OnCancel()
{
  if (installingThread != NULL) {
    TerminateThread(installingThread->m_hThread, -1);
    MessageBox("Installation cancelled.\n"
               "Run the Uninstall program to undo any partial installation.\n",
               "Information", MB_OK);
    installingThread = NULL;
  }
  
  CDialog::OnCancel();
}

// CInstallDlg::OnBrowse is called when the Browse button is clicked,
// to allow the user to browse for a new main RightSpeed directory.
// If the user selects a directory that does not end in \RightSpeed,
// it appends \RightSpeed to it. Finally, it updates the directory
// name in the dialog box and dialog variable.

void CInstallDlg::OnBrowse() 
{
  BROWSEINFO bi;
  LPITEMIDLIST finalIdList;
  char str[MAX_PATH];
  BOOL success;
  int length;

  bi.hwndOwner = m_hWnd;
  bi.pidlRoot = NULL;
  bi.pszDisplayName = str;
  bi.lpszTitle = "Select main RightSpeed directory";
  bi.ulFlags = BIF_RETURNONLYFSDIRS;
  bi.lpfn = NULL;
  bi.lParam = NULL;
  bi.iImage = NULL;

  finalIdList = SHBrowseForFolder(&bi);
  if (finalIdList == NULL) return;
  success = SHGetPathFromIDList(finalIdList, str);

  // The user has successfully browsed for a folder at this point,
  // and that folder name is in the string "str".

  if (success) {
    // If it does not already end in \RightSpeed, make it end so

    length = strlen(str);
    if (length < 11 || stricmp(&str[length-11], "\\RightSpeed")) {
      if (str[length-1] == '\\')
        strcat(str, "RightSpeed");
      else
        strcat(str, "\\RightSpeed");
    }
    SetDlgItemText(IDC_EDIT1, str);
    UpdateData(TRUE);
  }

  // SHFree(finalIdList);
}

// CInstallDlg::OnFreeSpace is called when the Check Free Space button is
// clicked.  It reports to the user the amount of free space in the chosen
// directory.

void CInstallDlg::OnFreeSpace()
{
  UINT freeMB;
  CString str;

  UpdateData(TRUE);

  // Check how much free space is in the directory

  if (GetFreeMB(m_mainDirectory, &freeMB)) {
    str.Format("%s has %d MB of free space.", m_mainDirectory, freeMB);
    MessageBox(str, "Free Space", MB_OK);
  }
  else
    DisplayError("Could not get free disk space");
}

// OnChangeStatus is called when a WM_CHANGE_STATUS message is posted to the
// dialog box.  The wParam field is a pointer to a string that the status
// message should be changed to.

void CInstallDlg::OnChangeStatus (WPARAM wParam, LPARAM lParam)
{
  SetDlgItemText(IDC_STATIC_PROGRESS, (char *) wParam);
}

// OnInstallComplete is called when a WM_INSTALL_COMPLETE message is posted to
// the dialog box.  It indicates that the installing thread is finished.  So,
// we clear the variable installingThread so we don't try to terminate it
// later.

void CInstallDlg::OnInstallComplete (WPARAM wParam, LPARAM lParam)
{
  installingThread = NULL;
  OnCancel();
}

/////////////////////////////////////////////////////////////////////////////
//                              FIELD CHECKING
/////////////////////////////////////////////////////////////////////////////

// CheckFields checks whether the fields supplied by the user are valid
// for doing an install.

BOOL CInstallDlg::CheckFields (void)
{
  char testFileName[MAX_PATH];
  UINT freeMB;
  WIN32_FIND_DATA findData;
  HANDLE findHandle;

  // Make sure the directory to install to either is the same as the
  // former directory or doesn't exist or is empty.

  if (m_mainDirectory != m_formerMainDirectory) {
    sprintf(testFileName, "%s\\*", m_mainDirectory);
    findHandle = FindFirstFile(testFileName, &findData);
    if (findHandle != INVALID_HANDLE_VALUE) {
      do {
        if (findData.cFileName[0] != '.' ||
            !((findData.cFileName[1] == '.' &&
               findData.cFileName[2] == '\0') ||
              (findData.cFileName[1] == '\0'))) {
          FindClose(findHandle);
          MessageBox("Cannot install to a non-empty directory.");
          return FALSE;
        }
      } while (FindNextFile(findHandle, &findData));
      FindClose(findHandle);
    }
  }

  // Check to for sufficient free space

  if (GetFreeMB(m_mainDirectory, &freeMB)) {
    if (freeMB < 2) {
      MessageBox("Fewer than 2 MB free.  Cannot install.", "Error",
                 MB_OK | MB_ICONEXCLAMATION);
      return FALSE;
    }
  }

  // Check to make sure max string lengths are not violated

  if (m_mainDirectory.GetLength() + 1 > MAX_RIGHTSPEED_DIRECTORY_LENGTH) {
    MessageBox("The main directory path is too long.\n"
               "Please choose a directory with a shorter name.");
    return FALSE;
  }

  // Check that main directory isn't being changed

  if (m_formerMainDirectory.GetLength() > 0 &&
      m_formerMainDirectory.CompareNoCase(m_mainDirectory)) {
    MessageBox(
      "You can't change the main directory using the installer "
      "since RightSpeed is already installed to a different directory.\n"
      "You have two choices: (1) Leave the directory the same. "
      "(2) Uninstall RightSpeed first, then reboot, then install it "
      "with the new directory.");
    return FALSE;
  }

  return TRUE;
}
