// This file contains information about global variables in the
// RSIoCnt driver.

#include "../Common/DrvShare.h"

////////////////////////////////////////////////////////////////////////////
//                                MACROS
////////////////////////////////////////////////////////////////////////////

// The following flags are used to keep track of which resources have
// been allocated so far for the RSIoCnt driver. Thus, when we exit,
// we can free these resources.

#define MAIN_DEVICE_RESOURCE_FLAG                1
#define SYMBOLIC_LINK_RESOURCE_FLAG              2
#define REGISTER_FS_CHANGE_RESOURCE_FLAG         4
#define RSINIT_FILE_OBJECT_RESOURCE_FLAG         8

////////////////////////////////////////////////////////////////////////////
//                             STRUCTURES
////////////////////////////////////////////////////////////////////////////

// This enumeration describes the different types of device we create:

typedef enum _our_device_type {
  OUR_DEVICE_TYPE_MAIN,
  OUR_DEVICE_TYPE_FS_RECOGNIZER_FILTER,
  OUR_DEVICE_TYPE_FS_FILTER
} OUR_DEVICE_TYPE;

// This structure will be used as the device extension for all our
// devices:

typedef struct _our_device_extension {
  OUR_DEVICE_TYPE ourDeviceType;
  PDEVICE_OBJECT filteredDevice;
} OUR_DEVICE_EXTENSION, *POUR_DEVICE_EXTENSION;

// The structure RS_IOCNT_GLOBALS contains all the global variables
// used by this driver in one place, so it's clear what's global and
// what isn't.

typedef struct _rs_iocnt_globals {
  // Miscellaneous fields:

  // Resources that must be freed before exiting:
  ULONG                        resourcesObtained;

  // Our driver object:
  PDRIVER_OBJECT               ourDriver;

  // The state shared with all RightSpeed drivers:
  SHARED_STATE                 *sharedState;

  // Our interface/notification device object:
  PDEVICE_OBJECT               mainDevice;

  // A file object pointer that references RSInit and makes sure it
  // sticks around:
  PFILE_OBJECT                 RSInitFileObject;

  // A lock controlling modifications to what devices our devices are
  // filtering.
  ERESOURCE                    deviceFilteringLock;

} RS_IOCNT_GLOBALS;

/////////////////////////////////////////////////////////////////////////
//                         EXTERNAL VARIABLES
/////////////////////////////////////////////////////////////////////////

extern RS_IOCNT_GLOBALS g;

