// This file contains definitions relevant to the log buffer.

#ifndef _RSLOG_LOG_BUFFER_H
#define _RSLOG_LOG_BUFFER_H

// This structure contains a description of the log buffer.

typedef struct _log_buffer_info {

  // This is the length of the buffer allocated for the log:
  ULONG bufferSize;
  // This is a pointer to the buffer allocated for the log:
  PCHAR allocatedSpace;
  // This is a pointer to the end of the buffer allocated for the log:
  PCHAR endOfAllocatedSpace;
  // This is a pointer to the start of the unflushed bytes:
  PCHAR startPos;
  // This is a pointer to the end of the unflushed bytes:
  PCHAR endPos;
  // This is the number of unflushed bytes, *including* the empty
  // space at the end of the log buffer:
  ULONG unflushedBytes;
  // This is a boolean indicating if the unflushed bytes are wrapped
  // around the end of the buffer:
  BOOLEAN wrapped;
  // This is the number of bytes of empty space at the end of the
  // buffer:
  ULONG emptySpaceAtEnd;
  // This is the number of unflushed bytes at which we trigger a flush:
  ULONG dangerousLength;

} LOG_BUFFER_INFO, *PLOG_BUFFER_INFO;

NTSTATUS InitializeLogBuffer (PLOG_BUFFER_INFO info);
void FreeLogBuffer (PLOG_BUFFER_INFO info);

PCHAR LogEvent (short eventType, ULONG dataLength);
NTSTATUS FlushLogBytes (
  HANDLE fileHandle,
  ULONG bytesToFlush,
  BOOLEAN wrapped
  );

#endif
