#ifndef _RIGHTSPEED_KERNEL_LOGGING_H
#define _RIGHTSPEED_KERNEL_LOGGING_H

#include "../Common/Logging.h"
#include "../Common/Schedule.h"

////////////////////////////////////////////////////////////////////////////
// LOG CATEGORY CHECKS
////////////////////////////////////////////////////////////////////////////

// The following macros determine whether logging of a certain category
// of information is currently active:

#define LOGGING_ERRORS \
  (g.sharedState->kernelUserSharedState.loggedEventCategorySet & \
   RS_EVENT_CATEGORY_ERRORS)
#define LOGGING_WARNINGS \
  (g.sharedState->kernelUserSharedState.loggedEventCategorySet & \
   RS_EVENT_CATEGORY_WARNINGS)
#define LOGGING_INFORMATION \
 (g.sharedState->kernelUserSharedState.loggedEventCategorySet & \
  RS_EVENT_CATEGORY_INFORMATION)
#define LOGGING_SPEEDS \
  (g.sharedState->kernelUserSharedState.loggedEventCategorySet & \
   RS_EVENT_CATEGORY_SPEEDS)
#define LOGGING_TASKS \
 (g.sharedState->kernelUserSharedState.loggedEventCategorySet & \
  RS_EVENT_CATEGORY_TASKS)
#define LOGGING_IO \
  (g.sharedState->kernelUserSharedState.loggedEventCategorySet & \
   RS_EVENT_CATEGORY_IO)
#define LOGGING_SCHEDULES \
  (g.sharedState->kernelUserSharedState.loggedEventCategorySet & \
   RS_EVENT_CATEGORY_SCHEDULES)

////////////////////////////////////////////////////////////////////////////
// RSLOG ROUTINES
////////////////////////////////////////////////////////////////////////////

void LogAnyEventType (short eventType);
void LogAnyEventTypeAndBytes (short eventType, PVOID data, ULONG dataLength);
void LogAnyEventTypeAndLong (short eventType, long n);
void LogString (char *str);
void LogSchedule (PSPEED_SCHEDULE schedule);

void LogError (
  NTSTATUS errorCode
  );

void LogErrorOneParameter (
  NTSTATUS errorCode,
  ULONG param1
  );

void LogErrorTwoParameters (
  NTSTATUS errorCode,
  ULONG param1,
  ULONG param2
  );

void LogErrorThreeParameters (
  NTSTATUS errorCode,
  ULONG param1,
  ULONG param2,
  ULONG param3
  );

////////////////////////////////////////////////////////////////////////////
// SYSTEM LOGGING ROUTINES
////////////////////////////////////////////////////////////////////////////

void SysLog (
  PVOID DeviceOrDriverObject,
  NTSTATUS ErrorCode,
  NTSTATUS FinalStatus,
  ULONG UniqueErrorValue,
  UCHAR MajorFunctionCode,
  ULONG IoControlCode,
  USHORT NumDumpDataEntries,
  ULONG *DumpData
  );

void SysLogShort (
  NTSTATUS ErrorCode,
  NTSTATUS FinalStatus
  );

void SysLogShortRepeat (
  NTSTATUS ErrorCode,
  NTSTATUS FinalStatus,
  ULONG UniqueErrorValue
  );

void SysLogOneParameter (
  NTSTATUS ErrorCode,
  NTSTATUS FinalStatus,
  ULONG DumpParameter
  );

void SysLogTwoParameters (
  NTSTATUS ErrorCode,
  NTSTATUS FinalStatus,
  ULONG DumpParameter0,
  ULONG DumpParameter1
  );

////////////////////////////////////////////////////////////////////////////
// COMBINED RSLOG AND SYSTEM LOGGING ROUTINES
////////////////////////////////////////////////////////////////////////////

void BothLogShort (
  NTSTATUS ErrorCode,
  NTSTATUS FinalStatus
  );

void BothLogShortRepeat (
  NTSTATUS ErrorCode,
  NTSTATUS FinalStatus,
  ULONG UniqueErrorValue
  );

void BothLogOneParameter (
  NTSTATUS ErrorCode,
  NTSTATUS FinalStatus,
  ULONG DumpParameter
  );

void BothLogTwoParameters (
  NTSTATUS ErrorCode,
  NTSTATUS FinalStatus,
  ULONG DumpParameter0,
  ULONG DumpParameter1
  );

#endif
