#ifndef _JRL_ARRAYS_H
#define _JRL_ARRAYS_H

namespace jrl {

  // SafeArray is an array that automatically gets deleted when it goes out
  // of scope.

  template <class T> class SafeArray {
  private:
    SafeArray(const SafeArray& b);
    SafeArray& operator= (const SafeArray& b);
    T* buf;

  public:
    SafeArray(int length) { buf = new T[length]; }
    ~SafeArray() { delete[] buf; }

    operator T* () { return buf; }
    T& operator[] (int n) { return buf[n]; }
  };

  // GrowArray is an array that automatically resizes itself when needed,
  // and that automatically gets deleted when it goes out of scope.

  template <class T> class GrowArray {
  public:
    GrowArray(int originalSize = 8)
      : size(originalSize), values(new T[originalSize]) { }
    ~GrowArray() { delete[] values; }
    T& operator[] (int n);
    int GetSize() const { return size; }

  private:
    int size;
    T* values;
  };

  template <class T> T& GrowArray<T>::operator[] (int n)
  {
    if (n >= size) {
      int newSize;
      for (newSize = size * 2; n >= newSize; newSize *= 2)
        ;
      T *newValues = new T[newSize];
      memcpy(newValues, values, sizeof(T)*size);
      delete[] values;
      size = newSize;
      values = newValues;
    }

    return values[n];
  }

}

#endif
