// This file contains routines for performing mathematical conversions.

#include <ntddk.h>
#include "../Common/Drivers.h"
#include "Convert.h"
#include "Globals.h"

////////////////////////////////////////////////////////////////////////////
// MACROS
////////////////////////////////////////////////////////////////////////////

// This is the conversion factor one multiplies by to convert a speed
// in KHz to one in cycles per 100 nanoseconds:
#define KHZ_TO_CYC_PER_HNS            1.0e-4

// This is the number of hundreds of nanoseconds per millisecond:
#define HNS_PER_MS                    10000

////////////////////////////////////////////////////////////////////////////
// CONVERSION
////////////////////////////////////////////////////////////////////////////

// This function converts a number in hundreds of nanoseconds into one
// in intervals. It returns the positive integer closest to the true
// value.

long HnsToIntervals (ULONG hns)
{
  long intervals;

  // Divide by the scheduling interval, rounding to the nearest integer.

  intervals = (hns + g.timerResolutionInfo.halfSchedulingIntervalHns) /
              g.timerResolutionInfo.schedulingIntervalHns;

  // Return the result, but nothing less than 1.
  
  if (intervals < 1)
    intervals = 1;
  return intervals;
}

// This function converts a timestamp count into a number of hundreds of
// nanoseconds.

LONGLONG TSCToHns (LONGLONG tsc)
{
  tsc *= HNS_PER_MS;
  tsc += (g.registryInfo.tscTicksPerMs >> 1);
  return tsc / g.registryInfo.tscTicksPerMs;
}

// This function converts a speed in cycles per milliseconds into one
// in cycles per scheduling interval.

double CycPerMsToCpi (ULONG cycPerMs)
{
  return cycPerMs * KHZ_TO_CYC_PER_HNS *
         g.timerResolutionInfo.schedulingIntervalHns;
}
