// This file contains definitions for using certain floating point
// operations that aren't enabled by default in the DDK.

#ifndef _MY_FLOAT_H
#define _MY_FLOAT_H

/////////////////////////////////////////////////////////////////////////////
// MACROS
/////////////////////////////////////////////////////////////////////////////

// These macros represent rounding modes that can be used in floating-
// point operations:

#define ROUNDING_MODE_TO_NEAREST          0x0
#define ROUNDING_MODE_DOWN                0x4
#define ROUNDING_MODE_UP                  0x8
#define ROUNDING_MODE_TOWARD_ZERO         0xc

/////////////////////////////////////////////////////////////////////////////
// FUNCTIONS
/////////////////////////////////////////////////////////////////////////////

char GetDefaultRoundingMode (void);
void SetDefaultRoundingMode (char mode);

long RoundDown (double x);
long RoundUp (double x);
long RoundTowardZero (double x);
long RoundToNearest (double x);
long RoundDefault (double x);

double SquareRoot (double x);
double TwoToThePowerOf (double x);
double Exponentiate (double base, double exponent);

#endif
