// This file contains definitions related to managing the set of ongoing
// tasks.

#ifndef _RSTASK_ONGOING_TASKS_H
#define _RSTASK_ONGOING_TASKS_H

#include "TTGFile.h"

/////////////////////////////////////////////////////////////////////////////
// MACROS
/////////////////////////////////////////////////////////////////////////////

// This is the maximum number of ongoing tasks we keep track of:
#define MAX_NUM_ONGOING_TASKS             16

/////////////////////////////////////////////////////////////////////////////
// STRUCTURES
/////////////////////////////////////////////////////////////////////////////

// This structure contains information about the set of ongoing tasks:

typedef struct _ongoing_tasks_info {

  // This spin lock protects access to all fields in this structure
  // as well as to the global numOngoingTasks in g.sharedState:
  KSPIN_LOCK spinLock;

  // This is the last task ID assigned to a task:
  long lastTaskId;

  // This is an array of the ID's of all the ongoing tasks. The first
  // 'numOngoingTasks' elements of this array represent those ID's,
  // in the order in which they began.
  long ongoingTaskIds[MAX_NUM_ONGOING_TASKS];

  // This boolean indicates four things:
  //   (a) 'numOngoingTasks' is 1,
  //   (b) no other task began while this single ongoing task was ongoing
  //   (c) the remaining fields in this structure (ttInfo, ttgFile, and
  //       ioOccurredDuringTask) are valid and apply to this single
  //       ongoing task, and
  //   (d) this structure holds a reference to the TTG file pointed to
  //       by ttgFile.
  BOOLEAN ttInfoApplies;

  // This field points to the task type information for the single
  // ongoing task:
  PTT_INFO ttInfo;

  // This field points to the task type group of which the single ongoing
  // task's type is a part. When 'ttInfoApplies' is true in this structure,
  // this field is not only valid but this file is referenced by this
  // structure.
  PTTG_FILE ttgFile;

  // This field indicates whether the CPU was idle waiting for I/O
  // at any point while the single ongoing task was ongoing:
  BOOLEAN ioOccurredDuringTask;

} ONGOING_TASKS_INFO, *PONGOING_TASKS_INFO;

/////////////////////////////////////////////////////////////////////////////
// FUNCTION PROTOTYPES
/////////////////////////////////////////////////////////////////////////////

void InitializeOngoingTasks (PONGOING_TASKS_INFO info);
void CloseOngoingTasks (PONGOING_TASKS_INFO info);

NTSTATUS BeginTask (PTTG_FILE ttgFile, PTT_INFO ttInfo, long *taskIdPtr);
NTSTATUS EndTask (long taskId);
void EndAllTasks (void);

#endif
