// This file contains definitions for global classes, classes whose
// objects represent all global information in this library.

#ifndef _RSLIB_GLOBALS_H
#define _RSLIB_GLOBALS_H

#include "RSInitHandle.h"
#include "Hooks.h"

////////////////////////////////////////////////////////////////////////////
// STRUCTURES
////////////////////////////////////////////////////////////////////////////

// This class is instantiated once for all processes to hold all
// systemwide global information.

typedef struct _systemwide_globals {
  SystemwideGlobals();

  // The following booleans indicate whether the shift and control
  // keys are down:
  bool shiftKeyDown;
  bool ctrlKeyDown;

  // This boolean indicates whether we should disable all calls
  // to begin and end tasks:
  bool disableTasks;

} SystemwideGlobals;

// This class is instantiated once per process to hold all global
// information for that process.

struct Globals {
  Globals();
  ~Globals() { Close(); }

  bool Initialize ();
  void Close ();

  // Our open handle to RSInit:
  RSInitHandle initDeviceHandle;

  // Information about message hooks for this process:
  Hooks hooks;

  // Whether this structure was initialized OK when the library was
  // loaded:
  bool initializedOK;
};

extern Globals g;
extern SystemwideGlobals systemwideGlobals;

#endif
