// This file defines the class representing an open handle to the
// RSTask device. Some such handles correspond to task type group
// files.

#ifndef _RSLIB_RSTASK_HANDLE_H
#define _RSLIB_RSTASK_HANDLE_H

#include "../Common/TTG.h"

// This class represents an open handle to the RSTask device.

class RSTaskHandle {
public:
  RSTaskHandle();
  RSTaskHandle(const char *fileName);
  RSTaskHandle(const HANDLE& hVal);

  BOOL Open (const char *fileName);
  BOOL AllowFileModification ();
  BOOL Flush ();
  void Close ();

  operator HANDLE () const { return h; }
  operator = (const HANDLE& hVal) { h = hVal; }

  DWORD BeginTask (DWORD TaskType);
  BOOL EndTask (DWORD TaskId);
  DWORD EndTaskAndBeginAnother (DWORD TaskId, DWORD TaskType);
  BOOL QueryDeadline (DWORD TaskType, double *DeadlinePtr);
  BOOL ChangeDeadline (DWORD TaskType, double Deadline);
  BOOL ResetSample (DWORD TaskType);
  BOOL ResetAllSamples ();
  BOOL QueryNumSpeeds (DWORD *NumSpeedsPtr);
  DWORD QueryTTGDirectory (char *buf, DWORD length);
  BOOL QueryTTGInfoPreamble (PTTG_INFO ttgInfo);
  BOOL SetGlobalPolicy (DWORD policyType, DWORD policyValue);
  BOOL QueryAvailableSpeedsKHz (DWORD *AvailableSpeedsPtr, DWORD NumSpeeds);

private:
  // This is the actual handle to the RSTask device:

  HANDLE h;
};

#endif

