// This file contains the main functionality for TTGRead, a program for
// reading and printing a description of a task type group file.

#include "stdafx.h"
#include <windows.h>
#include <iostream>
#include <string>
#include "exceptions.h"
#include "globals.h"
#include "TTGFile.h"

// ReadTTGFile reads and prints the contents of the task type
// group file of the given name.

void ReadTTGFile (string fileName)
{
  TTGFile ttgFile;

  try {
    ttgFile.Open(fileName);
    ttgFile.Print(std::cout);
  }
  catch (Exceptions::InvalidTTGFileFormat) {
    cerr << "ERROR - Invalid task type group file format" << endl;
  }
  catch (Exceptions::VersionNotSupported) {
    cerr << "ERROR - Version not supported" << endl;
  }
  catch (Exceptions::OpenFailed) {
    cerr << "ERROR - Open failed" << endl;
  }
}

// ReadFile reads and prints the contents of the file of the given name.

void ReadFile (string fileName)
{
  int length = fileName.size();
  if (length < 4) throw Exceptions::BadFileName();

  string extension = fileName.substr(length-3);

  if (extension == "ttg")
    ReadTTGFile(fileName);
  else
    throw Exceptions::BadFileName();
}

// The main program handles all arguments, treating some as parameters and
// some as file names.

int main (int argc, char **argv)
{
  if (argc < 2) return 0;

  for (int i = 1; i < argc; i++) {
    try {
      if (argv[i][0] == '-' && argv[i][1] != '.')
        g.HandleParameter(&argv[i][1]);
      else
        ReadFile(argv[i]);
    }
    catch (Exceptions::BadFileName) {
      cerr << "ERROR - Bad file name " << argv[i] << endl;
    }
  }

  return 0;
}
