#ifndef _TYPE_CONVERSIONS_H
#define _TYPE_CONVERSIONS_H

#include <windows.h>

// This file declares functions for converting among character buffer
// representations and numeric types. It also defines the LONGLONG and
// ULONGLONG types, which represent 8-byte numeric values.

namespace jrl {

  // Here we define the 8-byte types LONGLONG and ULONGLONG. These
  // typedefs allow porting to machines that don't have 8-byte
  // representations; on those machines, we can make LONGLONG and
  // ULONGLONG be classes supporting all the numeric operations we do
  // on them.

  // The following functions generate 8-byte values:

  inline ULONGLONG MakeUlonglong (unsigned long high, unsigned long low)
  { return (((ULONGLONG) high) << 32) + low; }

  inline LONGLONG MakeLonglong (long high, unsigned long low)
  { return (((LONGLONG) high) << 32) + low; }

  inline ULONGLONG MaxUlonglong ()
  { return MakeUlonglong(0xFFFFFFFFUL, 0xFFFFFFFFUL); }

  inline LONGLONG MaxLonglong ()
  { return MakeLonglong(0x7FFFFFFF, 0xFFFFFFFFUL); }

  // The following declarations declare functions that convert from
  // little endian character buffer representations to numeric types:

  LONGLONG Convert5BytesToLonglong (const char *buf);
  ULONGLONG Convert5BytesToUlonglong (const char *buf);
  LONGLONG Convert6BytesToLonglong (const char *buf);
  ULONGLONG Convert6BytesToUlonglong (const char *buf);
  double ConvertToDouble (const char *buf);
  LONGLONG ConvertToLonglong (const char *buf);
  ULONGLONG ConvertToUlonglong (const char *buf);
  long ConvertToLong (const char *buf);
  unsigned long ConvertToUlong (const char *buf);
  short ConvertToShort (const char *buf);
  unsigned short ConvertToUshort (const char *buf);

  // The following declarations declare functions that convert from
  // numeric types to little endian character buffer representations:

  void ConvertDouble (double number, char *buf);
  void ConvertLonglong (LONGLONG number, char *buf);
  void ConvertUlonglong (ULONGLONG number, char *buf);
  void ConvertLong (long number, char *buf);
  void ConvertUlong (unsigned long number, char *buf);
  void ConvertShort (short number, char *buf);
  void ConvertUshort (unsigned short number, char *buf);

  // The following declarations declare functions that convert between
  // numeric types and big endian character buffer representations:

  unsigned long ConvertToUlongBigEndian (const char *buf);
  unsigned short ConvertToUshortBigEndian (const char *buf);
  void ConvertUlongBigEndian (unsigned long number, char *buf);
  void ConvertUshortBigEndian (unsigned short number, char *buf);

}

#endif
