//
// UninstallDlg.cpp : implements the CUninstallDlg class
//

#include "stdafx.h"
#include "Uninstall.h"
#include "UninstallDlg.h"
#include <winioctl.h>
#include <winsvc.h>
#include "UninstallRoutines.h"
#include "../Common/Version.h"
#include "../Common/Parameters.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CUninstallDlg dialog

CUninstallDlg::CUninstallDlg(CWnd* pParent /*=NULL*/)
  : CDialog(CUninstallDlg::IDD, pParent)
{
  //{{AFX_DATA_INIT(CUninstallDlg)
    // NOTE: the ClassWizard will add member initialization here
  //}}AFX_DATA_INIT
  // Note that LoadIcon does not require a subsequent DestroyIcon in Win32
  m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CUninstallDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  //{{AFX_DATA_MAP(CUninstallDlg)
    // NOTE: the ClassWizard will add DDX and DDV calls here
  //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CUninstallDlg, CDialog)
  //{{AFX_MSG_MAP(CUninstallDlg)
  ON_WM_SYSCOMMAND()
  ON_WM_PAINT()
  ON_WM_QUERYDRAGICON()
  //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CUninstallDlg message handlers

BOOL CUninstallDlg::OnInitDialog()
{
  CDialog::OnInitDialog();

  // Set the icon for this dialog.  The framework does this automatically
  //  when the application's main window is not a dialog
  SetIcon(m_hIcon, TRUE);     // Set big icon
  SetIcon(m_hIcon, FALSE);    // Set small icon
  
  // TODO: Add extra initialization here
  
  return TRUE;  // return TRUE  unless you set the focus to a control
}

// The system calls this to obtain the cursor to display while the user drags
// the minimized window.
HCURSOR CUninstallDlg::OnQueryDragIcon()
{
  return (HCURSOR) m_hIcon;
}

// CUninstallDlg::OnOK is called when the user hits OK.  It uninstalls
// RightSpeed.

void CUninstallDlg::OnOK() 
{
  SC_HANDLE schSCManager;
  char systemRoot[MAX_PATH], mainDirectory[MAX_RIGHTSPEED_DIRECTORY_LENGTH];
  BOOLEAN completeSuccess = TRUE;
  int response;
  CWnd *okButton;

  // Disable the OK button

  okButton = GetDlgItem(IDOK);
  if (okButton != NULL) okButton->EnableWindow(FALSE);

  // Request confirmation of the request

  response = MessageBox("Are you sure you want to uninstall RightSpeed?",
                        "Uninstall", MB_YESNO | MB_DEFBUTTON2);
  if (response != IDYES) {
    CDialog::OnOK();
    return;
  }

  // Get the name of the RightSpeed main directory, needed for various
  // later things.

  if (!GetMainDirectory(mainDirectory, MAX_RIGHTSPEED_DIRECTORY_LENGTH))
    return;

  // Get system root directory name, needed for various later things

  if (GetEnvironmentVariable("SystemRoot", systemRoot, MAX_PATH) == 0) {
    DisplayError("Could not get SystemRoot environment variable");
    return;
  }

  // Prepare RSTask for uninstallation.

  if (!PrepareRSTaskForUninstall()) {
    DisplayError("Could not prepare RSTask for uninstallation");
    return;
  }

  // Delete the DLL and all references to it

  if (!DeleteDLL(systemRoot)) completeSuccess = FALSE;

  // Open service control manager in order to delete RightSpeed services

  schSCManager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
  if (schSCManager == NULL)
    DisplayError("Could not open service control manager to delete "
                 "RightSpeed services");
  else {
    // Delete drivers

    if (!DeleteDriver("RSTask",
                      schSCManager,
                      systemRoot,
                      FALSE)) // don't stop it first
      completeSuccess = FALSE;
    if (!DeleteDriver("RSIocnt",
                      schSCManager,
                      systemRoot,
                      FALSE)) // don't stop it first
      completeSuccess = FALSE;
    if (!DeleteDriver("RSLog",
                      schSCManager,
                      systemRoot,
                      TRUE)) // stop it first
      completeSuccess = FALSE;
    if (!DeleteDriver("RSInit",
                      schSCManager,
                      systemRoot,
                      FALSE)) // don't stop it first
      completeSuccess = FALSE;

    // Close handle to service control manager

    CloseServiceHandle(schSCManager);
  }

  // Delete the main RightSpeed directory and all its contents.

  if (completeSuccess && !DeleteMainDirectory(mainDirectory))
    completeSuccess = FALSE;

  // Remove shortcuts in Start menu.

  if (completeSuccess && !DeleteStartMenuShortcuts())
    completeSuccess = FALSE;

  // Delete the registry info associated with RightSpeed.

  if (completeSuccess && !DeleteRegistryInfo())
    completeSuccess = FALSE;

  // Display summary message.

  if (completeSuccess)
    MessageBox("RightSpeed uninstalled.\n"
               "You must restart the machine for some aspects of the "
               "uninstallation to be complete.",
               "Uninstall");
  else
    MessageBox("RightSpeed not completely uninstalled.", "Uninstall");

  CDialog::OnOK();
}
